package com.printer.sample;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;
import com.printer.aidl.PService;
import com.printer.command.EscCommand;
import com.printer.command.PrinterCom;
import com.printer.command.TscCommand;
import com.printer.io.PrinterDevice;
import com.printer.service.PrinterService;
import com.sample.R;
import org.apache.commons.lang.ArrayUtils;

import java.util.Vector;

public class MainActivity extends Activity {
  private              PService                 mPService      = null;
  public static final  String                   CONNECT_STATUS = "connect.status";
  private static final String                   DEBUG_TAG      = "MainActivity";
  private              PrinterServiceConnection conn           = null;
  private              int                      mPrinterIndex  = 0;
  private              int                      mTotalCopies   = 0;

  class PrinterServiceConnection implements ServiceConnection {
    @Override
    public void onServiceDisconnected(ComponentName name) {
      Log.i("ServiceConnection", "onServiceDisconnected() called");
      mPService = null;
    }

    @Override
    public void onServiceConnected(ComponentName name, IBinder service) {
      mPService = PService.Stub.asInterface(service);
    }
  }

  ;

  @Override
  protected void onResume() {
    // TODO Auto-generated method stub
    super.onResume();
    Log.e(DEBUG_TAG, "onResume");
  }

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);
    Log.e(DEBUG_TAG, "onCreate");

    connection();
  }

  private void connection() {
    conn = new PrinterServiceConnection();
    Intent intent = new Intent("com.printer.aidl.PrinterService");
    bindService(intent, conn, Context.BIND_AUTO_CREATE); // bindService
  }

  public boolean[] getConnectState() {
    boolean[] state = new boolean[PrinterService.MAX_PRINTER_CNT];
    for (int i = 0; i < PrinterService.MAX_PRINTER_CNT; i++) {
      state[i] = false;
    }
    for (int i = 0; i < PrinterService.MAX_PRINTER_CNT; i++) {
      try {
        if (mPService.getPrinterConnectStatus(i) == PrinterDevice.STATE_CONNECTED) {
          state[i] = true;
        }
      } catch (RemoteException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
    return state;
  }

  public void openPortDialogueClicked(View view) {
    Log.d(DEBUG_TAG, "openPortConfigurationDialog ");
    Intent intent = new Intent(this,
        PrinterConnectDialog.class);
    boolean[] state = getConnectState();
    intent.putExtra(CONNECT_STATUS, state);
    this.startActivity(intent);
  }

  public void printTestPageClicked(View view) {
    try {
      int rel = mPService.printeTestPage(mPrinterIndex); //
      Log.i("ServiceConnection", "rel " + rel);
      PrinterCom.ERROR_CODE r = PrinterCom.ERROR_CODE.values()[rel];
      if (r != PrinterCom.ERROR_CODE.SUCCESS) {
        Toast.makeText(getApplicationContext(), PrinterCom.getErrorText(r),
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
  }

  public void getPrinterStatusClicked(View view) {
    try {
      mTotalCopies = 0;
      int status = mPService.queryPrinterStatus(mPrinterIndex, 500);
      String str = new String();
      if (status == PrinterCom.STATE_NO_ERR) {
        str = "打印机正常";
      } else {
        str = "打印机 ";
        if ((byte) (status & PrinterCom.STATE_OFFLINE) > 0) {
          str += "脱机";
        }
        if ((byte) (status & PrinterCom.STATE_PAPER_ERR) > 0) {
          str += "缺纸";
        }
        if ((byte) (status & PrinterCom.STATE_COVER_OPEN) > 0) {
          str += "打印机开盖";
        }
        if ((byte) (status & PrinterCom.STATE_ERR_OCCURS) > 0) {
          str += "打印机出错";
        }
        if ((byte) (status & PrinterCom.STATE_TIMES_OUT) > 0) {
          str += "查询超时";
        }
      }
      Toast.makeText(getApplicationContext(),
          "打印机：" + mPrinterIndex + " 状态：" + str, Toast.LENGTH_SHORT).show();
    } catch (RemoteException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
  }

  public void getPrinterCommandTypeClicked(View view) {
    try {
      int type = mPService.getPrinterCommandType(mPrinterIndex);
      if (type == PrinterCom.ESC_COMMAND) {
        Toast.makeText(getApplicationContext(), "打印机使用ESC命令",
            Toast.LENGTH_SHORT).show();
      } else {
        Toast.makeText(getApplicationContext(), "打印机使用TSC命令",
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
  }

  void sendReceipt() {
    EscCommand esc = new EscCommand();
    esc.addPrintAndFeedLines((byte) 3);
    esc.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);//设置打印居中
    esc.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.ON, EscCommand.ENABLE.ON, EscCommand.ENABLE.OFF);//设置为倍高倍宽
    esc.addText("Sample\n");   //  打印文字
    esc.addPrintAndLineFeed();

		/*打印文字*/
    esc.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF);//取消倍高倍宽
    esc.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);//设置打印左对齐
    esc.addText("Print text\n");   //  打印文字
    esc.addText("Welcome to use printer!\n");   //  打印文字

		/*打印繁体中文  需要打印机支持繁体字库*/
    String message = Util.SimToTra("佳博票据打印机\n");
    //	esc.addText(message,"BIG5");
    esc.addText(message, "GB2312");
    esc.addPrintAndLineFeed();

		/*打印图片*/
    esc.addText("Print bitmap!\n");   //  打印文字
    Bitmap b = BitmapFactory.decodeResource(getResources(),
        R.drawable.ic_launcher);
    esc.addRastBitImage(b, 640, 0);   //打印图片
  
		/*打印一维条码*/
    esc.addText("Print code128\n");   //  打印文字
    esc.addSelectPrintingPositionForHRICharacters(EscCommand.HRI_POSITION.BELOW);//设置条码可识别字符位置在条码下方
    esc.addSetBarcodeHeight((byte) 60); //设置条码高度为60点
    esc.addCODE128("printer");  //打印Code128码
    esc.addPrintAndLineFeed();
  
		/*QRCode命令打印
      此命令只在支持QRCode命令打印的机型才能使用。
			在不支持二维码指令打印的机型上，则需要发送二维条码图片
		*/
//		esc.addText("Print QRcode\n");   //  打印文字	
//		esc.addSelectErrorCorrectionLevelForQRCode((byte)0x31); //设置纠错等级
//		esc.addSelectSizeOfModuleForQRCode((byte)3);//设置qrcode模块大小
//		esc.addStoreQRCodeData("www.Printer.com.cn");//设置qrcode内容
//		esc.addPrintQRCode();//打印QRCode
//		esc.addPrintAndLineFeed();

		/*打印文字*/
    esc.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);//设置打印左对齐
    esc.addText("Completed!\r\n");   //  打印结束
    esc.addPrintAndFeedLines((byte) 8);

    Vector<Byte> datas = esc.getCommand(); //发送数据
    Byte[] Bytes = datas.toArray(new Byte[datas.size()]);
    byte[] bytes = ArrayUtils.toPrimitive(Bytes);
    String str = Base64.encodeToString(bytes, Base64.DEFAULT);
    int rel;
    try {
      rel = mPService.sendEscCommand(mPrinterIndex, str);
      PrinterCom.ERROR_CODE r = PrinterCom.ERROR_CODE.values()[rel];
      if (r != PrinterCom.ERROR_CODE.SUCCESS) {
        Toast.makeText(getApplicationContext(), PrinterCom.getErrorText(r),
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  void sendReceipt3() {
    EscCommand esc = new EscCommand();
    esc.addText("1234567890\n");   //  打印文字
    Vector<Byte> datas = esc.getCommand(); //发送数据
    Byte[] Bytes = datas.toArray(new Byte[datas.size()]);
    byte[] bytes = ArrayUtils.toPrimitive(Bytes);
    String str = Base64.encodeToString(bytes, Base64.DEFAULT);
    int rel;
    try {
      rel = mPService.sendEscCommand(mPrinterIndex, str);
      PrinterCom.ERROR_CODE r = PrinterCom.ERROR_CODE.values()[rel];
      if (r != PrinterCom.ERROR_CODE.SUCCESS) {
        Toast.makeText(getApplicationContext(), PrinterCom.getErrorText(r),
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  void sendReceipt(int i) {
    EscCommand esc = new EscCommand();
    esc.addPrintAndFeedLines((byte) 3);
    esc.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);//设置打印居中
    esc.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.ON, EscCommand.ENABLE.ON, EscCommand.ENABLE.OFF);//设置为倍高倍宽
    esc.addText("Sample\n");   //  打印文字
    esc.addPrintAndLineFeed();

		/*打印文字*/
    esc.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF);//取消倍高倍宽
    esc.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);//设置打印左对齐
    esc.addText("Print text\n");   //  打印文字
    esc.addText("Welcome to use Printer!\n");   //  打印文字

		/*打印繁体中文  需要打印机支持繁体字库*/
    String message = Util.SimToTra("佳博票据打印机\n");
    //	esc.addText(message,"BIG5");
    esc.addText(message, "GB2312");
    esc.addPrintAndLineFeed();
		
		/*打印图片*/
    esc.addText("Print bitmap!\n");   //  打印文字
    Bitmap b = BitmapFactory.decodeResource(getResources(),
        R.drawable.printer);
    esc.addRastBitImage(b, b.getWidth(), 0);   //打印图片
	
		/*打印一维条码*/
    esc.addText("Print code128\n");   //  打印文字
    esc.addSelectPrintingPositionForHRICharacters(EscCommand.HRI_POSITION.BELOW);//设置条码可识别字符位置在条码下方
    esc.addSetBarcodeHeight((byte) 60); //设置条码高度为60点
    esc.addCODE128("Printer");  //打印Code128码
    esc.addPrintAndLineFeed();
	
		/*QRCode命令打印
			此命令只在支持QRCode命令打印的机型才能使用。
			在不支持二维码指令打印的机型上，则需要发送二维条码图片
		*/
    esc.addText("Print QRcode\n");   //  打印文字
    esc.addSelectErrorCorrectionLevelForQRCode((byte) 0x31); //设置纠错等级
    esc.addSelectSizeOfModuleForQRCode((byte) 3);//设置qrcode模块大小
    esc.addStoreQRCodeData("www.Printer.com.cn");//设置qrcode内容
    esc.addPrintQRCode();//打印QRCode
    esc.addPrintAndLineFeed();

    esc.addText("第 " + i + " 份\n");   //  打印文字
		/*打印文字*/
    esc.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);//设置打印左对齐
    esc.addText("Completed!\r\n");   //  打印结束
    esc.addPrintAndFeedLines((byte) 8);

    Vector<Byte> datas = esc.getCommand(); //发送数据
    Byte[] Bytes = datas.toArray(new Byte[datas.size()]);
    byte[] bytes = ArrayUtils.toPrimitive(Bytes);
    String str = Base64.encodeToString(bytes, Base64.DEFAULT);
    int rel;
    try {
      rel = mPService.sendEscCommand(mPrinterIndex, str);
      PrinterCom.ERROR_CODE r = PrinterCom.ERROR_CODE.values()[rel];
      if (r != PrinterCom.ERROR_CODE.SUCCESS) {
        Toast.makeText(getApplicationContext(), PrinterCom.getErrorText(r),
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  void sendReceiptBmp(int i) {
    EscCommand esc = new EscCommand();
		/*打印图片*/
    esc.addText("Print bitmap!\n");   //  打印文字
    Bitmap b = BitmapFactory.decodeResource(getResources(),
        R.drawable.test);
    esc.addRastBitImage(b, 384, 0);   //打印图片
    esc.addText("第 " + i + " 份\n");   //  打印文字

    Vector<Byte> datas = esc.getCommand(); //发送数据
    Byte[] Bytes = datas.toArray(new Byte[datas.size()]);
    byte[] bytes = ArrayUtils.toPrimitive(Bytes);
    String str = Base64.encodeToString(bytes, Base64.DEFAULT);
    int rel;
    try {
      rel = mPService.sendEscCommand(mPrinterIndex, str);
      PrinterCom.ERROR_CODE r = PrinterCom.ERROR_CODE.values()[rel];
      if (r != PrinterCom.ERROR_CODE.SUCCESS) {
        Toast.makeText(getApplicationContext(), PrinterCom.getErrorText(r),
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  void sendLabel() {
    TscCommand tsc = new TscCommand();
    tsc.addSize(60, 60); //设置标签尺寸，按照实际尺寸设置
    tsc.addGap(0);           //设置标签间隙，按照实际尺寸设置，如果为无间隙纸则设置为0
    tsc.addDirection(TscCommand.DIRECTION.BACKWARD, TscCommand.MIRROR.NORMAL);//设置打印方向
    tsc.addReference(0, 0);//设置原点坐标
    tsc.addTear(EscCommand.ENABLE.ON); //撕纸模式开启
    tsc.addCls();// 清除打印缓冲区
    //绘制简体中文
    tsc.addText(20, 20, TscCommand.FONTTYPE.SIMPLIFIED_CHINESE, TscCommand.ROTATION.ROTATION_0, TscCommand.FONTMUL.MUL_1, TscCommand.FONTMUL.MUL_1, "Welcome to use Printer!");
    //绘制图片
    Bitmap b = BitmapFactory.decodeResource(getResources(),
        R.drawable.printer);
    tsc.addBitmap(20, 50, TscCommand.BITMAP_MODE.OVERWRITE, b.getWidth() * 2, b);

    tsc.addQRCode(250, 80, TscCommand.EEC.LEVEL_L, 5, TscCommand.ROTATION.ROTATION_0, " www.Printer.com.cn");
    //绘制一维条码
    tsc.add1DBarcode(20, 250, TscCommand.BARCODETYPE.CODE128, 100, TscCommand.READABEL.EANBEL, TscCommand.ROTATION.ROTATION_0, "Printer");
    tsc.addPrint(1, 1); // 打印标签
    tsc.addSound(2, 100); //打印标签后 蜂鸣器响
    Vector<Byte> datas = tsc.getCommand(); //发送数据
    Byte[] Bytes = datas.toArray(new Byte[datas.size()]);
    byte[] bytes = ArrayUtils.toPrimitive(Bytes);
    String str = Base64.encodeToString(bytes, Base64.DEFAULT);
    int rel;
    try {
      rel = mPService.sendTscCommand(mPrinterIndex, str);
      PrinterCom.ERROR_CODE r = PrinterCom.ERROR_CODE.values()[rel];
      if (r != PrinterCom.ERROR_CODE.SUCCESS) {
        Toast.makeText(getApplicationContext(), PrinterCom.getErrorText(r),
            Toast.LENGTH_SHORT).show();
      }
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  public void printReceiptClicked(View view) {
    try {
      int type = mPService.getPrinterCommandType(mPrinterIndex);
      if (type == PrinterCom.ESC_COMMAND) {
        int status = mPService.queryPrinterStatus(mPrinterIndex, 500);
        if (status == PrinterCom.STATE_NO_ERR) {
          sendReceipt();
        } else {
          Toast.makeText(getApplicationContext(),
              "打印机错误！", Toast.LENGTH_SHORT).show();
        }
      }
    } catch (RemoteException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
  }

  public void printLabelClicked(View view) {
    try {
      int type = mPService.getPrinterCommandType(mPrinterIndex);
      if (type == PrinterCom.TSC_COMMAND) {
        int status = mPService.queryPrinterStatus(mPrinterIndex, 500);
        if (status == PrinterCom.STATE_NO_ERR) {
          sendLabel();
        } else {
          Toast.makeText(getApplicationContext(),
              "打印机错误！", Toast.LENGTH_SHORT).show();
        }
      }
    } catch (RemoteException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
  }

  public void printTestClicked(View view) {
    try {
      int type = mPService.getPrinterCommandType(mPrinterIndex);
      if (type == PrinterCom.ESC_COMMAND) {
        EditText etCopies = (EditText) findViewById(R.id.etPrintCopies);
        String str = etCopies.getText().toString();
        int copies = 0;
        if (!str.equals("")) {
          copies = Integer.parseInt(str);
        }
        mTotalCopies += copies;
        for (int i = 0; i < copies; i++) {
          sendReceipt();
        }
      }
    } catch (RemoteException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
  }

  @Override
  public void onDestroy() {
    Log.e(DEBUG_TAG, "onDestroy");
    super.onDestroy();
    if (conn != null) {
      unbindService(conn); // unBindService
    }
  }

}
